@@ -1,29 +1,31 @@
-from itertools import chain
-
-from plone.memoize.instance import memoize, clearafter
-from zope.component import getUtilitiesFor, getMultiAdapter
-from zope.i18n import translate
-from zope.event import notify
-
-from Acquisition import aq_parent, aq_base
 from AccessControl import Unauthorized
-from zExceptions import Forbidden
-
-from Products.CMFCore.utils import getToolByName
+from Acquisition import aq_base
+from Acquisition import aq_parent
+from itertools import chain
+from plone.app.workflow import PloneMessageFactory as _
+from plone.app.workflow.events import LocalrolesModifiedEvent
+from plone.app.workflow.interfaces import ISharingPageRole
+from plone.base.utils import safe_text
+from plone.i18n.normalizer.interfaces import IIDNormalizer
+from plone.memoize.instance import clearafter
+from plone.memoize.instance import memoize
 from Products.CMFCore import permissions
-from Products.CMFPlone.utils import normalizeString, safe_unicode
+from Products.CMFCore.utils import getToolByName
 from Products.Five.browser import BrowserView
 from Products.Five.browser.pagetemplatefile import ViewPageTemplateFile
 from Products.statusmessages.interfaces import IStatusMessage
-
-from plone.app.workflow import PloneMessageFactory as _
-from plone.app.workflow.interfaces import ISharingPageRole
-from plone.app.workflow.events import LocalrolesModifiedEvent
+from zExceptions import Forbidden
+from zope.component import getMultiAdapter
+from zope.component import getUtilitiesFor
+from zope.component import getUtility
+from zope.event import notify
+from zope.i18n import translate
 
 import json
 
-AUTH_GROUP = 'AuthenticatedUsers'
-STICKY = (AUTH_GROUP, )
+
+AUTH_GROUP = "AuthenticatedUsers"
+STICKY = (AUTH_GROUP,)
 
 
 def merge_search_results(results, key):
@@ -31,15 +33,15 @@ def merge_search_results(results, key):
 
     Based on PlonePAS.browser.search.PASSearchView.merge.
     """
-    output={}
+    output = {}
     for entry in results:
-        id=entry[key]
+        id = entry[key]
         if id not in output:
-            output[id]=entry.copy()
+            output[id] = entry.copy()
         else:
-            buf=entry.copy()
+            buf = entry.copy()
             buf.update(output[id])
-            output[id]=buf
+            output[id] = buf
 
     return output.values()
 
@@ -48,20 +50,18 @@ class SharingView(BrowserView):
 
     # Actions
 
-    template = ViewPageTemplateFile('sharing.pt')
-    macro_wrapper = ViewPageTemplateFile('macro_wrapper.pt')
+    template = ViewPageTemplateFile("sharing.pt")
+    macro_wrapper = ViewPageTemplateFile("macro_wrapper.pt")
 
     STICKY = STICKY
 
     def __call__(self):
-        """Perform the update and redirect if necessary, or render the page
-        """
+        """Perform the update and redirect if necessary, or render the page"""
         postback = self.handle_form()
         if postback:
             return self.template()
         else:
-            context_state = self.context.restrictedTraverse(
-                "@@plone_context_state")
+            context_state = self.context.restrictedTraverse("@@plone_context_state")
             url = context_state.view_url()
             self.request.response.redirect(url)
 
@@ -73,43 +73,44 @@ class SharingView(BrowserView):
         postback = True
 
         form = self.request.form
-        submitted = form.get('form.submitted', False)
-        save_button = form.get('form.button.Save', None) is not None
-        cancel_button = form.get('form.button.Cancel', None) is not None
+        submitted = form.get("form.submitted", False)
+        save_button = form.get("form.button.Save", None) is not None
+        cancel_button = form.get("form.button.Cancel", None) is not None
         if submitted and save_button and not cancel_button:
-            if not self.request.get('REQUEST_METHOD', 'GET') == 'POST':
+            if not self.request.get("REQUEST_METHOD", "GET") == "POST":
                 raise Forbidden
 
-            authenticator = self.context.restrictedTraverse('@@authenticator',
-                                                            None)
+            authenticator = self.context.restrictedTraverse("@@authenticator", None)
             if not authenticator.verify():
                 raise Forbidden
 
             # Update the acquire-roles setting
             if self.can_edit_inherit():
-                inherit = bool(form.get('inherit', False))
+                inherit = bool(form.get("inherit", False))
                 reindex = self.update_inherit(inherit, reindex=False)
             else:
                 reindex = False
 
             # Update settings for users and groups
-            entries = form.get('entries', [])
-            roles = [r['id'] for r in self.roles()]
+            entries = form.get("entries", [])
+            roles = [r["id"] for r in self.roles()]
             settings = []
             for entry in entries:
                 settings.append(
-                    dict(id=entry['id'],
-                         type=entry['type'],
-                         roles=[r for r in roles
-                            if entry.get('role_%s' % r, False)]))
+                    dict(
+                        id=entry["id"],
+                        type=entry["type"],
+                        roles=[r for r in roles if entry.get("role_%s" % r, False)],
+                    )
+                )
             if settings:
-                reindex = self.update_role_settings(settings, reindex=False) \
-                            or reindex
+                reindex = self.update_role_settings(settings, reindex=False) or reindex
             if reindex:
                 self.context.reindexObjectSecurity()
                 notify(LocalrolesModifiedEvent(self.context, self.request))
             IStatusMessage(self.request).addStatusMessage(
-                _(u"Changes saved."), type='info')
+                _("Changes saved."), type="info"
+            )
 
         # Other buttons return to the sharing page
         if cancel_button:
@@ -129,7 +130,7 @@ class SharingView(BrowserView):
             - title
         """
         context = self.context
-        portal_membership = getToolByName(context, 'portal_membership')
+        portal_membership = getToolByName(context, "portal_membership")
 
         pairs = []
 
@@ -139,12 +140,17 @@ class SharingView(BrowserView):
                 if not portal_membership.checkPermission(permission, context):
                     continue
             # be friendly to utilities implemented without required_interface
-            iface = getattr(utility, 'required_interface', None)
+            iface = getattr(utility, "required_interface", None)
             if iface is not None and not iface.providedBy(context):
                 continue
-            pairs.append(dict(id = name, title = utility.title))
+            pairs.append(dict(id=name, title=utility.title))
 
-        pairs.sort(key=lambda x: normalizeString(translate(x["title"], context=self.request)))
+        normalizer = getUtility(IIDNormalizer)
+        pairs.sort(
+            key=lambda x: normalizer.normalize(
+                translate(x["title"], context=self.request)
+            )
+        )
         return pairs
 
     @memoize
@@ -172,17 +178,16 @@ class SharingView(BrowserView):
         # We may be called when the user does a search instead of an update.
         # In that case we must not loose the changes the user made and
         # merge those into the role settings.
-        requested = self.request.form.get('entries', None)
+        requested = self.request.form.get("entries", None)
         if requested is not None:
-            knownroles = [r['id'] for r in self.roles()]
+            knownroles = [r["id"] for r in self.roles()]
             settings = {}
             for entry in requested:
-                roles = [r for r in knownroles
-                                if entry.get('role_%s' % r, False)]
-                settings[(entry['id'], entry['type'])] = roles
+                roles = [r for r in knownroles if entry.get("role_%s" % r, False)]
+                settings[(entry["id"], entry["type"])] = roles
 
             for entry in current_settings:
-                desired_roles = settings.get((entry['id'], entry['type']), None)
+                desired_roles = settings.get((entry["id"], entry["type"]), None)
 
                 if desired_roles is None:
                     continue
@@ -190,7 +195,9 @@ class SharingView(BrowserView):
                     if entry["roles"][role] in [True, False]:
                         entry["roles"][role] = role in desired_roles
 
-        current_settings.sort(key=lambda x: safe_unicode(x["type"])+safe_unicode(x["title"]))
+        current_settings.sort(
+            key=lambda x: safe_text(x["type"]) + safe_text(x["title"])
+        )
 
         return current_settings
 
@@ -201,13 +208,10 @@ class SharingView(BrowserView):
         return True
 
     def inherited(self, context=None):
-        """Return True if local roles are inherited here.
-        """
+        """Return True if local roles are inherited here."""
         if context is None:
             context = self.context
-        if getattr(aq_base(context), '__ac_local_roles_block__', None):
-            return False
-        return True
+        return not getattr(aq_base(context), "__ac_local_roles_block__", None)
 
     # helper functions
 
@@ -220,9 +224,9 @@ class SharingView(BrowserView):
         """
         context = self.context
 
-        portal_membership = getToolByName(context, 'portal_membership')
-        portal_groups = getToolByName(context, 'portal_groups')
-        acl_users = getToolByName(context, 'acl_users')
+        portal_membership = getToolByName(context, "portal_membership")
+        portal_groups = getToolByName(context, "portal_groups")
+        acl_users = getToolByName(context, "acl_users")
 
         info = []
 
@@ -230,53 +234,59 @@ class SharingView(BrowserView):
 
         local_roles = acl_users._getLocalRolesForDisplay(context)
         acquired_roles = self._inherited_roles() + self._borg_localroles()
-        available_roles = [r['id'] for r in self.roles()]
+        available_roles = [r["id"] for r in self.roles()]
 
         # first process acquired roles
         items = {}
         for name, roles, rtype, rid in acquired_roles:
-            items[rid] = dict(id = rid,
-                              name = name,
-                              type = rtype,
-                              sitewide = [],
-                              acquired = roles,
-                              local = [], )
+            items[rid] = dict(
+                id=rid,
+                name=name,
+                type=rtype,
+                sitewide=[],
+                acquired=roles,
+                local=[],
+            )
 
         # second process local roles
         for name, roles, rtype, rid in local_roles:
             if rid in items:
-                items[rid]['local'] = roles
+                items[rid]["local"] = roles
             else:
-                items[rid] = dict(id = rid,
-                                  name = name,
-                                  type = rtype,
-                                  sitewide = [],
-                                  acquired = [],
-                                  local = roles, )
+                items[rid] = dict(
+                    id=rid,
+                    name=name,
+                    type=rtype,
+                    sitewide=[],
+                    acquired=[],
+                    local=roles,
+                )
 
         # Make sure we always get the authenticated users virtual group
         if AUTH_GROUP not in items:
-            items[AUTH_GROUP] = dict(id = AUTH_GROUP,
-                                     name = _(u'Logged-in users'),
-                                     type = 'group',
-                                     sitewide = [],
-                                     acquired = [],
-                                     local = [], )
+            items[AUTH_GROUP] = dict(
+                id=AUTH_GROUP,
+                name=_("Logged-in users"),
+                type="group",
+                sitewide=[],
+                acquired=[],
+                local=[],
+            )
 
         # If the current user has been given roles, remove them so that he
         # doesn't accidentally lock himself out.
 
         member = portal_membership.getAuthenticatedMember()
         if member.getId() in items:
-            items[member.getId()]['disabled'] = True
+            items[member.getId()]["disabled"] = True
 
         # Sort the list: first the authenticated users virtual group, then
         # all other groups and then all users, alphabetically
 
-        dec_users = [(a['id'] not in self.STICKY,
-                       a['type'],
-                       a['name'],
-                       a) for a in items.values()]
+        dec_users = [
+            (a["id"] not in self.STICKY, a["type"], a["name"], a)
+            for a in items.values()
+        ]
         dec_users.sort()
 
         # Add the items to the info dict, assigning full name if possible.
@@ -284,18 +294,20 @@ class SharingView(BrowserView):
 
         for d in dec_users:
             item = d[-1]
-            name = item['name']
-            rid = item['id']
+            name = item["name"]
+            rid = item["id"]
             login = rid
             global_roles = set()
 
-            if item['type'] == 'user':
+            if item["type"] == "user":
                 member = acl_users.getUserById(rid)
                 if member is not None:
-                    name = member.getProperty('fullname') or member.getUserName() or name
+                    name = (
+                        member.getProperty("fullname") or member.getUserName() or name
+                    )
                     global_roles = set(member.getRoles())
                     login = member.getUserName()
-            elif item['type'] == 'group':
+            elif item["type"] == "group":
                 g = portal_groups.getGroupById(rid)
                 name = g.getGroupTitleOrName()
                 login = None
@@ -303,41 +315,45 @@ class SharingView(BrowserView):
 
                 # This isn't a proper group, so it needs special treatment :(
                 if rid == AUTH_GROUP:
-                    name = _(u'Logged-in users')
+                    name = _("Logged-in users")
 
-            info_item = dict(id = item['id'],
-                             type = item['type'],
-                             title = name,
-                             disabled = item.get('disabled', False),
-                             roles = {})
+            info_item = dict(
+                id=item["id"],
+                type=item["type"],
+                title=name,
+                disabled=item.get("disabled", False),
+                roles={},
+            )
             if login != name:
-                info_item['login'] = login
+                info_item["login"] = login
 
             # Record role settings
             have_roles = False
             for r in available_roles:
                 if r in global_roles:
-                    info_item['roles'][r] = 'global'
-                elif r in item['acquired']:
-                    info_item['roles'][r] = 'acquired'
-                    have_roles = True # we want to show acquired roles
-                elif r in item['local']:
-                    info_item['roles'][r] = True
-                    have_roles = True # at least one role is set
+                    info_item["roles"][r] = "global"
+                elif r in item["acquired"]:
+                    info_item["roles"][r] = "acquired"
+                    have_roles = True  # we want to show acquired roles
+                elif r in item["local"]:
+                    info_item["roles"][r] = True
+                    have_roles = True  # at least one role is set
                 else:
-                    info_item['roles'][r] = False
+                    info_item["roles"][r] = False
 
             if have_roles or rid in self.STICKY:
                 info.append(info_item)
 
         return info
 
-    def _principal_search_results(self,
-                                  search_for_principal,
-                                  get_principal_by_id,
-                                  get_principal_title,
-                                  principal_type,
-                                  id_key):
+    def _principal_search_results(
+        self,
+        search_for_principal,
+        get_principal_by_id,
+        get_principal_title,
+        principal_type,
+        id_key,
+    ):
         """Return search results for a query to add new users or groups.
 
         Returns a list of dicts, as per role_settings().
@@ -358,18 +374,22 @@ class SharingView(BrowserView):
         """
         context = self.context
 
-        translated_message = translate(_(u"Search for user or group"),
-                context=self.request)
-        search_term = safe_unicode(self.request.form.get('search_term', None))
+        translated_message = translate(
+            _("Search for user or group"), context=self.request
+        )
+        search_term = safe_text(self.request.form.get("search_term", None))
         if not search_term or search_term == translated_message:
             return []
 
-        existing_principals = set([p['id'] for p in self.existing_role_settings()
-                                if p['type'] == principal_type])
-        empty_roles = dict([(r['id'], False) for r in self.roles()])
+        existing_principals = {
+            p["id"]
+            for p in self.existing_role_settings()
+            if p["type"] == principal_type
+        }
+        empty_roles = {r["id"]: False for r in self.roles()}
         info = []
 
-        hunter = getMultiAdapter((context, self.request), name='pas_search')
+        hunter = getMultiAdapter((context, self.request), name="pas_search")
         for principal_info in search_for_principal(hunter, search_term):
             principal_id = principal_info[id_key]
             if principal_id not in existing_principals:
@@ -380,16 +400,19 @@ class SharingView(BrowserView):
 
                 for r in principal.getRoles():
                     if r in roles:
-                        roles[r] = 'global'
+                        roles[r] = "global"
                 login = principal.getUserName()
-                if principal_type == 'group':
+                if principal_type == "group":
                     login = None
-                info.append(dict(id = principal_id,
-                                 title = get_principal_title(principal,
-                                                             principal_id),
-                                 login = login,
-                                 type = principal_type,
-                                 roles = roles))
+                info.append(
+                    dict(
+                        id=principal_id,
+                        title=get_principal_title(principal, principal_id),
+                        login=login,
+                        type=principal_type,
+                        roles=roles,
+                    )
+                )
         return info
 
     def user_search_results(self):
@@ -399,18 +422,30 @@ class SharingView(BrowserView):
         """
 
         def search_for_principal(hunter, search_term):
-            return merge_search_results(chain(*[hunter.searchUsers(**{field: search_term})
-                for field in ['name', 'fullname', 'email']]), 'userid')
+            return merge_search_results(
+                chain(
+                    *[
+                        hunter.searchUsers(**{field: search_term})
+                        for field in ["name", "fullname", "email"]
+                    ]
+                ),
+                "userid",
+            )
 
         def get_principal_by_id(user_id):
-            acl_users = getToolByName(self.context, 'acl_users')
+            acl_users = getToolByName(self.context, "acl_users")
             return acl_users.getUserById(user_id)
 
         def get_principal_title(user, default_title):
-            return user.getProperty('fullname') or user.getId() or default_title
+            return user.getProperty("fullname") or user.getId() or default_title
 
-        return self._principal_search_results(search_for_principal,
-            get_principal_by_id, get_principal_title, 'user', 'userid')
+        return self._principal_search_results(
+            search_for_principal,
+            get_principal_by_id,
+            get_principal_title,
+            "user",
+            "userid",
+        )
 
     def group_search_results(self):
         """Return search results for a query to add new groups.
@@ -419,18 +454,30 @@ class SharingView(BrowserView):
         """
 
         def search_for_principal(hunter, search_term):
-            return merge_search_results(chain(*[hunter.searchGroups(**{field:search_term})
-                for field in ['id', 'title']]), 'groupid')
+            return merge_search_results(
+                chain(
+                    *[
+                        hunter.searchGroups(**{field: search_term})
+                        for field in ["id", "title"]
+                    ]
+                ),
+                "groupid",
+            )
 
         def get_principal_by_id(group_id):
-            portal_groups = getToolByName(self.context, 'portal_groups')
+            portal_groups = getToolByName(self.context, "portal_groups")
             return portal_groups.getGroupById(group_id)
 
         def get_principal_title(group, _):
             return group.getGroupTitleOrName()
 
-        return self._principal_search_results(search_for_principal,
-            get_principal_by_id, get_principal_title, 'group', 'groupid')
+        return self._principal_search_results(
+            search_for_principal,
+            get_principal_by_id,
+            get_principal_title,
+            "group",
+            "groupid",
+        )
 
     def _inherited_roles(self):
         """Returns a tuple with the acquired local roles."""
@@ -439,13 +486,13 @@ class SharingView(BrowserView):
         if not self.inherited(context):
             return ()
 
-        portal = getToolByName(context, 'portal_url').getPortalObject()
+        portal = getToolByName(context, "portal_url").getPortalObject()
         result = []
         cont = True
         if portal != context:
             parent = aq_parent(context)
             while cont:
-                if not getattr(parent, 'acl_users', False):
+                if not getattr(parent, "acl_users", False):
                     break
                 userroles = parent.acl_users._getLocalRolesForDisplay(parent)
                 for user, roles, role_type, name in userroles:
@@ -472,7 +519,7 @@ class SharingView(BrowserView):
                     parent = aq_parent(parent)
 
         # Tuplize all inner roles
-        for pos in range(len(result)-1, -1, -1):
+        for pos in range(len(result) - 1, -1, -1):
             result[pos][1] = tuple(result[pos][1])
             result[pos] = tuple(result[pos])
 
@@ -492,8 +539,8 @@ class SharingView(BrowserView):
         borg_local_roles = pas._getAllLocalRoles(self.context)
         for principal, roles in editable_local_roles.items():
             borg_local_roles[principal] = [
-                r for r in borg_local_roles.get(principal, ())
-                if r not in roles]
+                r for r in borg_local_roles.get(principal, ()) if r not in roles
+            ]
             if not borg_local_roles[principal]:
                 del borg_local_roles[principal]
 
@@ -501,9 +548,9 @@ class SharingView(BrowserView):
         result = []
         for principal, roles in borg_local_roles.items():
             username = principal
-            userType = 'user'
+            userType = "user"
             if pas.getGroup(principal):
-                userType = 'group'
+                userType = "group"
             else:
                 user = pas.getUserById(principal)
                 if user:
@@ -520,15 +567,15 @@ class SharingView(BrowserView):
         are the same as the existing settings.
         """
         context = self.context
-        portal_membership = getToolByName(context, 'portal_membership')
+        portal_membership = getToolByName(context, "portal_membership")
 
         if not portal_membership.checkPermission(
-                                permissions.ModifyPortalContent, context):
+            permissions.ModifyPortalContent, context
+        ):
             raise Unauthorized
 
         block = not status
-        oldblock = bool(getattr(aq_base(context),
-                                '__ac_local_roles_block__', False))
+        oldblock = bool(getattr(aq_base(context), "__ac_local_roles_block__", False))
 
         if block == oldblock:
             return False
@@ -565,14 +612,16 @@ class SharingView(BrowserView):
         changed = False
         context = self.context
 
-        managed_roles = frozenset([r['id'] for r in self.roles()])
+        managed_roles = frozenset(r["id"] for r in self.roles())
         member_ids_to_clear = []
 
         for s in new_settings:
-            user_id = s['id']
+            user_id = s["id"]
 
-            existing_roles = frozenset(context.get_local_roles_for_userid(userid=user_id))
-            selected_roles = frozenset(s['roles'])
+            existing_roles = frozenset(
+                context.get_local_roles_for_userid(userid=user_id)
+            )
+            selected_roles = frozenset(s["roles"])
 
             relevant_existing_roles = managed_roles & existing_roles
 
@@ -607,15 +656,11 @@ class SharingView(BrowserView):
 
         return changed
 
-    def updateSharingInfo(self, search_term=''):
+    def updateSharingInfo(self, search_term=""):
         self.handle_form()
-        the_id = 'user-group-sharing'
+        the_id = "user-group-sharing"
         macro = self.template.macros[the_id]
-        res = self.macro_wrapper(the_macro=macro, instance=self.context,
-                                 view=self)
-        messages = self.context.restrictedTraverse('global_statusmessage')()
-        self.request.response.setHeader("Content-type","application/json")
-        return json.dumps({
-            'body': res,
-            'messages': messages
-        })
+        res = self.macro_wrapper(the_macro=macro, instance=self.context, view=self)
+        messages = self.context.restrictedTraverse("global_statusmessage")()
+        self.request.response.setHeader("Content-type", "application/json")
+        return json.dumps({"body": res, "messages": messages})
